
package org.mule.module.twilio.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.twilio.processors.MakeCallMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class MakeCallDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public MakeCallDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(MakeCallMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("accountSid")!= null)&&(!StringUtils.isBlank(element.getAttribute("accountSid")))) {
            builder.addPropertyValue("accountSid", element.getAttribute("accountSid"));
        }
        if ((element.getAttribute("from")!= null)&&(!StringUtils.isBlank(element.getAttribute("from")))) {
            builder.addPropertyValue("from", element.getAttribute("from"));
        }
        if ((element.getAttribute("to")!= null)&&(!StringUtils.isBlank(element.getAttribute("to")))) {
            builder.addPropertyValue("to", element.getAttribute("to"));
        }
        if ((element.getAttribute("url")!= null)&&(!StringUtils.isBlank(element.getAttribute("url")))) {
            builder.addPropertyValue("url", element.getAttribute("url"));
        }
        if ((element.getAttribute("applicationSid")!= null)&&(!StringUtils.isBlank(element.getAttribute("applicationSid")))) {
            builder.addPropertyValue("applicationSid", element.getAttribute("applicationSid"));
        }
        if (element.hasAttribute("method")) {
            builder.addPropertyValue("method", element.getAttribute("method"));
        }
        String fallbackCallbackFlowName = getAttributeValue(element, "fallback-flow-ref");
        if (fallbackCallbackFlowName!= null) {
            builder.addPropertyValue("fallbackCallbackFlow", new RuntimeBeanReference(fallbackCallbackFlowName));
        }
        String statusCallbackCallbackFlowName = getAttributeValue(element, "status-callback-flow-ref");
        if (statusCallbackCallbackFlowName!= null) {
            builder.addPropertyValue("statusCallbackCallbackFlow", new RuntimeBeanReference(statusCallbackCallbackFlowName));
        }
        if ((element.getAttribute("sendDigits")!= null)&&(!StringUtils.isBlank(element.getAttribute("sendDigits")))) {
            builder.addPropertyValue("sendDigits", element.getAttribute("sendDigits"));
        }
        if ((element.getAttribute("ifMachine")!= null)&&(!StringUtils.isBlank(element.getAttribute("ifMachine")))) {
            builder.addPropertyValue("ifMachine", element.getAttribute("ifMachine"));
        }
        if ((element.getAttribute("timeout")!= null)&&(!StringUtils.isBlank(element.getAttribute("timeout")))) {
            builder.addPropertyValue("timeout", element.getAttribute("timeout"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
