
package org.mule.module.twilio.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.module.twilio.processors.UpdateApplicationMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class UpdateApplicationDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public UpdateApplicationDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(UpdateApplicationMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if ((element.getAttribute("accountSid")!= null)&&(!StringUtils.isBlank(element.getAttribute("accountSid")))) {
            builder.addPropertyValue("accountSid", element.getAttribute("accountSid"));
        }
        if ((element.getAttribute("applicationSid")!= null)&&(!StringUtils.isBlank(element.getAttribute("applicationSid")))) {
            builder.addPropertyValue("applicationSid", element.getAttribute("applicationSid"));
        }
        if ((element.getAttribute("friendlyName")!= null)&&(!StringUtils.isBlank(element.getAttribute("friendlyName")))) {
            builder.addPropertyValue("friendlyName", element.getAttribute("friendlyName"));
        }
        if ((element.getAttribute("apiVersion")!= null)&&(!StringUtils.isBlank(element.getAttribute("apiVersion")))) {
            builder.addPropertyValue("apiVersion", element.getAttribute("apiVersion"));
        }
        if ((element.getAttribute("voiceUrl")!= null)&&(!StringUtils.isBlank(element.getAttribute("voiceUrl")))) {
            builder.addPropertyValue("voiceUrl", element.getAttribute("voiceUrl"));
        }
        if (element.hasAttribute("voiceMethod")) {
            builder.addPropertyValue("voiceMethod", element.getAttribute("voiceMethod"));
        }
        String voiceFallbackCallbackFlowName = getAttributeValue(element, "voice-fallback-flow-ref");
        if (voiceFallbackCallbackFlowName!= null) {
            builder.addPropertyValue("voiceFallbackCallbackFlow", new RuntimeBeanReference(voiceFallbackCallbackFlowName));
        }
        String statusCallbackCallbackFlowName = getAttributeValue(element, "status-callback-flow-ref");
        if (statusCallbackCallbackFlowName!= null) {
            builder.addPropertyValue("statusCallbackCallbackFlow", new RuntimeBeanReference(statusCallbackCallbackFlowName));
        }
        if ((element.getAttribute("voiceCallerIdLookup")!= null)&&(!StringUtils.isBlank(element.getAttribute("voiceCallerIdLookup")))) {
            builder.addPropertyValue("voiceCallerIdLookup", element.getAttribute("voiceCallerIdLookup"));
        }
        if ((element.getAttribute("smsUrl")!= null)&&(!StringUtils.isBlank(element.getAttribute("smsUrl")))) {
            builder.addPropertyValue("smsUrl", element.getAttribute("smsUrl"));
        }
        if (element.hasAttribute("smsMethod")) {
            builder.addPropertyValue("smsMethod", element.getAttribute("smsMethod"));
        }
        String smsFallbackCallbackFlowName = getAttributeValue(element, "sms-fallback-flow-ref");
        if (smsFallbackCallbackFlowName!= null) {
            builder.addPropertyValue("smsFallbackCallbackFlow", new RuntimeBeanReference(smsFallbackCallbackFlowName));
        }
        String smsStatusCallbackCallbackFlowName = getAttributeValue(element, "sms-status-callback-flow-ref");
        if (smsStatusCallbackCallbackFlowName!= null) {
            builder.addPropertyValue("smsStatusCallbackCallbackFlow", new RuntimeBeanReference(smsStatusCallbackCallbackFlowName));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
